<?php

/**
 * Class: BB Code Parser
 *
 * Parses a text document with BB code into HTML
 *
 * @author Dan Pupius
 * @version 1.1
 * @copyright 2004 Daniel Pupius <http://pupius.co.uk>
 * @filesource
 */
class BB_Parser {

	/**
	 * @var boolean Whether or not to allow images
	 * @access private
	 */
	var $_allow_images = false;

	/**
	 * @var boolean Whether or not to allow links
	 * @access private
	 */
	var $_allow_links = true;

	/**
	 * @var boolean Whether or not to allow formatting such as bold, italic, underline
	 * @access private
	 */
	var $_allow_formatting = true;

	/**
	 * @var boolean Whether to escape html characters before BB parse
	 * @access private
	 */
	var $_allow_html = false;

	/**
	 * @var boolean Whether to convert line breaks into html line breaks
	 * @access private
	 */
	var $_convert_nl2br = true;
	
	/**
	 * @var boolean Whether to wrap content in <p> tags
	 * @access private
	 */
	var $_force_paragraph = false;
	
	/**
	 * @var boolean Make quotes and apostrophe's pretty
	 * @access private
	 */
	var $_pretty_chars = true;


	/*****************************************************************************************/



	/**
	 * Setter and getter for allow images
	 * 
	 * @param boolean $val True/False
	 * @access public
	 */
	function set_allow_images($val) { $this->_allow_images = ($val)?true:false; }
	function get_allow_images($val) { return $this->_allow_images; }


	/**
	 * Setter and getter for allow links
	 * 
	 * @param boolean $val True/False
	 * @access public
	 */
	function set_allow_links($val) { $this->_allow_links = ($val)?true:false; }
	function get_allow_links($val) { return $this->_allow_links; }


	/**
	 * Setter and getter for allow formatting
	 * 
	 * @param boolean $val True/False
	 * @access public
	 */
	function set_allow_formatting($val) { $this->_allow_formatting = ($val)?true:false; }
	function get_allow_formatting($val) { return $this->_allow_formatting; }


	/**
	 * Setter and getter for allow html
	 * 
	 * @param boolean $val True/False
	 * @access public
	 */
	function set_allow_html($val) { $this->_allow_html = ($val)?true:false; }
	function get_allow_html($val) { return $this->_allow_html; }


	/**
	 * Setter and getter for convert nl2br
	 * 
	 * @param boolean $val True/False
	 * @access public
	 */
	function set_convert_nl2br($val) { $this->_convert_nl2br = ($val)?true:false; }
	function get_convert_nl2br($val) { return $this->_convert_nl2br; }

	/**
	 * Setter and getter for convert force_paragraph
	 * 
	 * @param boolean $val True/False
	 * @access public
	 */
	function set_para($val) { $this->_force_paragraph = ($val)?true:false; }
	function get_para($val) { return $this->_force_paragraph; }

	/*****************************************************************************************/

	/**
	 * Formats a string of code by adding line numbers to the start
	 * 
	 * @param string $code Code of some kind
	 * @return string formatted
	 * @access public
	 */
	function format_code($code) {
		$code = str_replace("&#8220;","&quot;",$code);
		$code = str_replace("&#8221;","&quot;",$code);
		$code = str_replace("&#8217;","'",$code);

		$lines = split("\n",$code);

		$count = 1;
		foreach($lines as $key => $val) {
			if($count < 10) $no = "<span class=\"linenumber\">$count</span>   ";
			else if($count < 100) $no = "<span class=\"linenumber\">$count</span>  ";
			else if($count < 1000) $no = "<span class=\"linenumber\">$count</span> ";

			$lines[$key] = $no . $val;			
			$count++;
		}
		return implode("\n",$lines);
	}


	/**
	 * Creates an image tag for an image
	 * 
	 * @param string $img Image URL (relative to webroot or absolute)
	 * @return string formatted
	 * @access public
	 */
	function create_imgtag($img,$class="") {
		if(strpos($img,"http://") === false) $url = $GLOBALS['url_base'] . $img;
		else $url = $img;

		$arr = getimagesize($url);

		if($class!="") $class = " class=\"$class\"";

		return "<img$class src=\"$img\" " . $arr[3] . " alt=\"$img\" />";

	}



	/**
	 * Parses a string based on current settings and returns a processed version
	 * 
	 * @param string $string String to process
	 * @return string Pricessed string
	 * @access public
	 */	
	function parse($string,$firstclass="") {



		//remove HTML
		if(!$this->_allow_html) $string = htmlentities($string);

		//convert line breaks
		if($this->_convert_nl2br) $string = preg_replace("/\n/im","<br />\n",$string);

		//make chars pretty before adding the html
		if($this->_pretty_chars) {
			$string = preg_replace("/\s*--\s*/"," &#8212; ",$string);
			$string = preg_replace("/([a-zA-Z0-9])'([a-zA-Z0-9])/","\\1&#8217;\\2",$string);
			$string = preg_replace("/([^a-zA-Z0-9])&quot;([\[a-zA-Z0-9])/","\\1&#8220;\\2",$string);
			$string = preg_replace("/([a-zA-Z0-9\]])&quot;([^a-zA-Z0-9])/","\\1&#8221;\\2",$string);
		}

		//do basic formatting
		if($this->_allow_formatting) {
			$string = preg_replace("(\[b\](.+?)\[\/b])is",'<strong>$1</strong>',$string); 
			$string = preg_replace("(\[i\](.+?)\[\/i])is",'<em>$1</em>',$string); 
			$string = preg_replace("(\[u\](.+?)\[\/u])is",'<span style=\"text-decoration:underline\">$1</span>',$string); 
			$string = preg_replace("(\[s\](.+?)\[\/s])is",'<span style=\"text-decoration:line-through\">$1</span>',$string); 
			$string = preg_replace("(\[size=(.+?)\](.+?)\[\/size\])is","<span style=\"font-size: $1px\">$2</span>",$string); 
	
			$string = preg_replace("/\[quote\](.+?)\[\/quote\]/is","</p>\n<p class=\"quote\">$1</p>\n<p>", $string);
			$string = preg_replace("/\[code\](.+?)\[\/code\]/ise","'</p>\n<pre class=\"code\">'.BB_Parser::format_code('$1').'</pre>\n<p>'", $string);
		}

		if($this->_allow_links) {
			$url_chars = " a-zA-Z0-9\:\/\-\?\&\.\=\_\~\#\'\%"; 
			$mail_chars = $url_chars . "@"; 			

			//convert urls
			$string = preg_replace("/\[url\](.*?)\[\/url\]/", '<a href="$1">$1</a>', $string); 
			$string = preg_replace("(\[url href\=(.*?)\](.*?)\[/url\])", '<a href="$1">$2</a>', $string);
			$string = preg_replace("(\[url class\=(.*?) href\=(.*?)\](.*?)\[/url\])", '<a class="$1" href="$2">$3</a>', $string); 

			
			//convert images
			$string = preg_replace("/\[img\](.+?)\[\/img\]/ise","BB_Parser::create_imgtag('$1')", $string); 
			$string = preg_replace("/\[img class\=(.*?)\](.+?)\[\/img\]/ise","BB_Parser::create_imgtag('$2','$1')", $string); 

			//convert emails
			//$string = preg_replace("/\[email\](.*?)\[\/email\]/", '<a class="email" href="mailto:$1">$1</a>', $string); 
			//$string = preg_replace("/([email to\=(.*?)\](.*?)\[/email\])e", "'<a class=\"email\" href=\"mailto:'.urlencode('$1').'\">$2</a>'", $string);
		}

		//make sure string is in a paragraph
		if($firstclass!="" && $this->_force_paragraph) $string = "\n<p class=\"$firstclass\">$string</p>\n\n";
		else if($this->_force_paragraph) $string = "\n<p>$string</p>\n\n";

		//but remove empty paragraphs just in case
		$string = str_replace("<p></p>","",$string);

		return $string;
	}


	/**
	 * Echos the parsed string
	 * 
	 * @param string $string String to process
	 * @access public
	 */	
	function output($string,$firstclass="") { echo $this->parse($string,$firstclass); }

}

/*
$bb = new BB_Parser;
$bb->output('
[email to=dan@pupius.co.uk]Email me[/email]
[email]dan@pupius.co.uk[/email]
[url class=external href=http://www.bbc.co.uk]BBC News[/url]
');
*/

?>
